#!/usr/bin/perl
#
# Jeremy Chartier, <jeremy.chartier@free.fr>
# Date: 2003/03/03
# Revision: 1.8
# 

use Getopt::Long;		# use Getopt for options
use Socket;                     # use socket for resolving domain name from IP

$domains_file = "domains";
$rules_file = "rules";
$addr_len = 15;
$nb_len = 6;
$port_len = 5;
$attack_len = 90;
$class_len = 60;
$resolve_len = 50;
$hour_len = 2;
$graph_len = 75;
$prior_len = 1;
$sever_len = 7;
$domain_len = 30;

$version = "1.8.0";
$datever = "2003/03/03 11:42:01";
$date = `date '+%Y/%m/%d at %H:%M:%S'`;

Getopt::Long;

GetOptions( \%opt,
	'd',
	'r',
	'i',
	'h',
	'n=i',
	'c',
	'class',
	'src',
	'dst',
	'class_attack',
	'attack',
	'dport',
	'dport_attack',
	'nids',
	'stateful',
	'severity',
	'src_attack',
	'dst_attack',
	'hour',
	'hour_attack',
	'domain_src',
	'src_dst_attack',
	'portscan',
	'priority'
	) or usage();
	

# process whatever comes in
while (<>) {
my $alert = {};
chomp;
# if the line is blank, go to the next one
next if $_ eq "";



if ( $_ =~ /^\[\*\*\]/ ) {
	if ( $_ =~ m/^\[\*\*\]\s+\[[0-9:]+\]\s\(spp_portscan\d?\)\sPortscan\sdetected\sfrom\s([\d\.]+)/ox) { # portscan
		$alert->{SADDR} = $1; $alert->{TYPE} = "plugin"; 
		$alert->{PLUGIN} = "portscan";
		process_data($alert); next;
	} else {
		s/\s*\[\*\*\]\s*//og;
		s/\s*\[[0-9:]+\]\s*//o;
		$SIG = $_;
		next;
	}
}


if ( $_ =~ m/^\[Classification\:([^\[|^\]]*?)\]\s+\[Priority\:\s+(\d{1})\]/ox ) {
	if ($2 == 1) {
		$severity = "high";
	} elsif ($2 == 2) {
		$severity = "medium";
	} elsif ($2 == 3) {
		$severity = "low";
	} else {
		$severity = "unknown"
	}
	$CLASS = $1;
	$PRIORITY = $2;
	$SEVERITY = $severity;
	next;
}




if ( $_ =~ /^(\d{2})\/(\d{2})\-(\d{2})\:(\d{2})\:(\d{2})\.\d+\s+([\d\.]+)\:?([\d]*)\s[\-\>]+\s([\d\.]+)\:?([\d]*)/) {
		$alert->{MON} = $1;   $alert->{DAY} = $2;   $alert->{HOUR} = $3;
		$alert->{MIN} = $4;   $alert->{SEC} = $5;   $alert->{SADDR} = $6;
		$alert->{SPORT} = $7;   $alert->{DADDR} = $8;   $alert->{DPORT} = $9;
		$alert->{HOST} = "localhost";
		$alert->{SIG} = $SIG;
		$alert->{TYPE} = "sys";
		$alert->{PLUGIN} = "none";
		$alert->{CLASS} = $CLASS;
		$alert->{PRIORITY} = $PRIORITY;
		$alert->{SEVERITY} = $SEVERITY;
		process_data($alert);
		next;
}




# is this line an alert message
if ( $_ =~ /\d{2}\/\d{2}\-\d{2}\:\d{2}\:\d{2}\.\d+\s+\[\*\*\]/ ) {
#	$line = <>;
#	chomp($line);
	unless ( $_ eq "" ) {
		# strip off the [**] from either end.
		s/\s*\[\*\*\]\s*//og;
		s/\s*\[[0-9:]+\]\s*//o;
		if ( $_ =~ m/^(\d+)\/(\d+)\-(\d+)\:(\d+)\:(\d+)\.(\d+)(.*)/ox ) {
#		   ([\d\.]+)[\:]*([\d]*)\s[\-\>]+\s([\d\.]+)[\:]*([\d]*)\s(.+)/ox) {
			$alert->{MON} = $1;   $alert->{DAY} = $2;   $alert->{HOUR} = $3; 
			$alert->{MIN} = $4;   $alert->{SEC} = $5;   $alert->{SIG} = $7; 
			$alert->{HOST} = "localhost";
			$alert->{SIG} =~ s/\s*\[[\d\:]+\]\s*//;  # Get rid of [343:33:31]
			$alert->{SIG} =~ s/\[\*\*\]//og; # Get rid of [**] if fast alert
			if ($alert->{SIG} =~ m/^\(spp_portscan\d?\)\sPortscan\sdetected\sfrom\s([\d\.]+)/ox) { # portscan
				$alert->{SADDR} = $1; $alert->{TYPE} = "plugin"; 
				$alert->{PLUGIN} = "portscan";
				process_data($alert); $lastwassnort = 1; next;
			} elsif ( $alert->{SIG} =~ s/\s([\d\.]+)[\:]?([\d]*)\s[\-\>]+\s
                           ([\d\.]+)[\:]?([\d]*)\s*//x) {
				$alert->{SADDR} = $1; $alert->{SPORT} = $2; 
				$alert->{DADDR} = $3; $alert->{DPORT} = $4; 
				if ($alert->{SIG} =~ m/spp_anomsensor\:\sAnomaly\sthreshold\s
					  exceeded\:\s([\d\.]+)/ox) { # spade
					$alert->{THR} = $1; $alert->{TYPE} = "plugin"; 
					$alert->{PLUGIN} = "anomsensor"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} elsif ($alert->{SIG} =~ s/spp_bo\:\s//ox) { # bo
					$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "bo"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} elsif ($alert->{SIG} =~ /spp_stream4/) { # stream4 
					$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "stream4"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} elsif ($alert->{SIG} =~ /spp_frag2/) { # frag2 
					$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "frag2"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} elsif ($alert->{SIG} =~ /snort_decoder/) { # snort_decoder 
					$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "snort_decoder"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} else { # normal alert
					if ( $alert->{SIG} =~ s/\[Classification\:([^\[|^\]]*?)\]\s*(?:\[Priority\:\s(\d)\])//x ) {
						if ($2 == 1) {
							$severity = "high";
						} elsif ($2 == 2) {
							$severity = "medium";
						} elsif ($2 == 3) {
							$severity = "low";
						} else {
							$severity = "unknown"
						}
						$alert->{CLASS} = $1; $alert->{PRIORITY} = $2; $alert->{SEVERITY} = $severity; 
					}
					$alert->{TYPE} = "sys"; $alert->{PLUGIN} = "none";
					process_data($alert); $lastwassnort = 1; next;
				}
			} else { 
				print STDERR "No source/dest IP address found! Skipped! --> $_\n" if $opt{d}; 
				next;
    			}
		}
	} 
}

# This is syslog format
if ($_ =~ m/^(\w{3}) \s+ (\d+) \s (\d+)\:(\d+)\:(\d+)\s 
                   (\S+?)\ssnort[\[\d+\]]*\:\s+(.+)/ox 
      || m/^(\d+)\/(\d+)\-(\d+)\:(\d+)\:(\d+)\.(\d+)\s(.+)/ox) 
{
	$alert->{MON} = $1;    $alert->{DAY} = $2;    $alert->{HOUR} = $3; 
	$alert->{MIN} = $4;    $alert->{SEC} = $5;    $alert->{HOST} = $6;  
	$alert->{SIG} = $7;    
	$alert->{SIG} =~ s/\s*\[[\d\:]+\]\s*//;  # Get rid of [343:33:31]
	$alert->{SIG} =~ s/\[\*\*\]//og; # Get rid of [**] if fast alert
	if ($alert->{SIG} =~ m/^\(spp_portscan\d?\)\sPortscan\sdetected\sfrom\s([\d\.]+)/ox || m/spp_portscan:\sPORTSCAN\sDETECTED\sfrom\s([\d\.]+)/ox) { # portscan
		$alert->{SADDR} = $1; $alert->{TYPE} = "plugin"; 
		$alert->{PLUGIN} = "portscan";
		process_data($alert); $lastwassnort = 1; next;
	} elsif ( $alert->{SIG} =~ s/\s([\d\.]+)[\:]?([\d]*)\s[\-\>]+\s
                           ([\d\.]+)[\:]?([\d]*)\s*//x) {
		$alert->{SADDR} = $1; $alert->{SPORT} = $2; 
		$alert->{DADDR} = $3; $alert->{DPORT} = $4; 
		if ($alert->{SIG} =~ m/spp_anomsensor\:\sAnomaly\sthreshold\s
			  exceeded\:\s([\d\.]+)/ox) { # spade
			$alert->{THR} = $1; $alert->{TYPE} = "plugin"; 
			$alert->{PLUGIN} = "anomsensor"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ s/spp_bo\:\s//ox) { # bo
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "bo"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ /spp_stream4/) { # stream4 
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "stream4"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ /spp_frag2/) { # frag2 
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "frag2"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ /spp_portscan/) { # frag2 
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "portscan"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ /snort_decoder/) { # snort_decoder 
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "snort_decoder"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} else { # normal alert
			if ( $alert->{SIG} =~ s/\[Classification\:([^\[|^\]]*?)\]\s*(?:\[Priority\:\s(\d)\]\:\s)//x ) {
				if ($2 == 1) {
					$severity = "high";
				} elsif ($2 == 2) {
					$severity = "medium";
				} elsif ($2 == 3) {
					$severity = "low";
				} else {
					$severity = "unknown"
				}
				$alert->{CLASS} = $1; $alert->{PRIORITY} = $2; $alert->{SEVERITY} = $severity;  
			}
			$alert->{TYPE} = "sys"; $alert->{PLUGIN} = "none";
			process_data($alert); $lastwassnort = 1; next;
		}
	} else {
		print STDERR "No source/dest IP address found! Skipped! --> $_\n" if $opt{d}; 
		$alert = {}; next;
	}
}
# If a snort message has been repeated several times
elsif ($lastwassnort && $_ =~ m/last message repeated (\d+) times/) {
# put the data in the matrix again for each repeat
	$repeats = $1;
	while ($repeats) {
		push @result, $result[-1];
		$repeats--;
	}
	next;
} else {
	$lastwassnort = 0;
	next;
}				# Message not related to snort
}

# begin statistics
# I should've used $#result + 1 as $total in the first version! :(
$total = $#result + 1;
$totalscan = $#portscan + 1;

if ($opt{n}) {
	$nb=$opt{n};
} else {	
	$nb=10000;
}

if ( $total == 0 ) {
	print "No Snort's logs found\n";
	exit
} else {
$report=$opt{report} if $opt{report};

for $i ( 0 .. $#result ) {
  # for the same pair of attacker and victim with same attack
  # used in same_src_dst_attack()
  $s0{"$result[$i]->[9],$result[$i]->[7],$result[$i]->[6]"}++;
  # used in same_src_attack()
  $s2{"$result[$i]->[6],$result[$i]->[7],$result[$i]->[14]"}++;
  # to same victim with same method
  # to see how many attacks received by one host
  # used in same_dest_attack()
  $s3{"$result[$i]->[6],$result[$i]->[9],$result[$i]->[14]"}++;
  # same signature
  # to see the popularity of one attack method
  # used in attack()
  $s4{"$result[$i]->[6],$result[$i]->[13],$result[$i]->[14]"}++;
  # used in attack_src()
  $s5{"$result[$i]->[7]"}++;
  # used by attack_dst 
  $s6{"$result[$i]->[9]"}++;
  # used by hour()
  $s7{"$result[$i]->[2]"}++;
  # use by same_hour_attack()
  $s9{"$result[$i]->[2],$result[$i]->[6]"}++;
  # to see classification
  # used by same_class() 
  $s10{"$result[$i]->[11],$result[$i]->[14]"}++;
  # use by nids()
  $s13{"$result[$i]->[5]"}++;
  # use by stateful()
  $s17{"$result[$i]->[12]"}++;
  # use by dport_attack()
  $s18{"$result[$i]->[10],$result[$i]->[6]"}++;
  # use by daily_event()
  $s19{"$result[$i]->[1],$result[$i]->[0]"}++;
  # use by severity()
  $s20{"$result[$i]->[14]"}++;
}
 

$processor = keys(%s17);
$anomaly = --$processor;

init_domains();
init_rules();
print_head();

if ( $opt{src} ) {
	attack_src();
} elsif ( $opt{dst} ) {
	attack_dst();
} elsif ( $opt{src_attack} ) {
	same_src_attack();
} elsif ( $opt{dst_attack} ) {
	same_dst_attack();
} elsif ( $opt{src_dst_attack} ) {
	same_src_dst_attack();
} elsif ( $opt{attack} ) {
	attack();
} elsif ( $opt{class} ) {
	same_class();
} elsif ( $opt{severity} ) {
	severity();
} elsif ( $opt{daily_event} ) {
	daily_event() if ($result[0]->[0] ne $result[$#result]->[0]) || ($result[0]->[1] < $result[$#result]->[1]);
} elsif ( $opt{hour} ) {
	hour();
} elsif ( $opt{hour_attack} ) {
	same_hour_attack();
} elsif ( $opt{dport_attack} ) {
	dport_attack();
} elsif ( $opt{nids} ) {
	nids() if (keys(%s13) > 1);
} elsif ( $opt{stateful} ) {
	stateful() if ($anomaly > 0);
} elsif ( $opt{domain_src} ) {
	domain_src();
} elsif ( $opt{portscan} ) {
	portscan() if ( $totalscan > 0 );
} else {
# begin report
#print_head();
print_summary();
print_menu() if $opt{h};
attack_src();
same_src_attack();
attack_dst();
same_dst_attack();
same_src_dst_attack();
attack();
same_class();
severity();
daily_event() if ($result[0]->[0] ne $result[$#result]->[0]) || ($result[0]->[1] < $result[$#result]->[1]);
hour();
same_hour_attack();
dport_attack();
nids() if (keys(%s13) > 1);
stateful() if ($anomaly > 0);
domain_src() if ( $opt{c} );
portscan() if ( $totalscan > 0 );
print_footer();
}
}


# print the header (e.g. for mail)
sub print_head {
if ($opt{h}) {
	print "<HTML>\n<HEAD>\n<STYLE>\n<!--\n";
	print "BODY { font-family: serif; background-color: white; font-size: 12px; }\n";
	print "/* Table Header */\n";
	print ".TH { font-family: Verdana,Arial,Helvetica,sans-serif; font-size: 12px; background-color: #FFEFC6; color: blue; font-weight: bold; }\n";
	print "/* Table body */\n";
	print ".TB { color: black; background-color: white; font-size: 12px; }\n";
	print "/* Table body RED*/\n";
	print ".TBR { color: red; background-color: white; font-size: 12px; }\n";
	print "/* Table body GREEN*/\n";
	print ".TBG { color: green; background-color: white; font-size: 12px; }\n";
	print "-->\n</STYLE>\n</HEAD>\n";
	print "<BODY>\n";
	print "<h1>IDS Statistics generated the $date</h1>\n";
} else {
	print "subject: ids report generated the $date\n"; 
}
}

# print the time of begin and end of the log
sub print_summary {
	if ($opt{h}) {
		print "<TABLE>\n";
		print "<UL><A NAME=\"top\"></A>\n";
		print "<TR ALIGN=LEFT><TH>The log begins at:</TH>\n";
		print "<TD>$result[0]->[0] $result[0]->[1] $result[0]->[2]:$result[0]->[3]:$result[0]->[4]</TD><TD WIDTH=170></TD><TD><B>Legende:</B></TD></TR>\n";
		print "<TR ALIGN=LEFT><TH>The log ends at:</TH>\n";
		print "<TD>$result[$#result]->[0] $result[$#result]->[1] $result[$#result]->[2]:$result[$#result]->[3]:$result[$#result]->[4]</TD><TD></TD><TD><FONT COLOR= RED>RED : Dangerous</FONT></TD></TR>\n";
		print "<TR ALIGN=LEFT><TH>Total events:</TH><TD> $total</TD><TD WIDTH=50></TD><TD><FONT COLOR= GREEN>GREEN : Warning</FONT></TD></TR>\n";
		print "<TR ALIGN=LEFT><TH>NIDS recorded:</TH><TD> ". keys(%s13) ."</TD><TD></TD><TD><FONT COLOR= BLACK>BLACK : Normal</FONT></TD></TR>\n";
		print "<TR ALIGN=LEFT><TH>Signatures recorded:</TH><TD> ". keys(%s4) ."</TD></TR>\n";
		print "<TR ALIGN=LEFT><TH>Classification recorded:</TH><TD> ". keys(%s10) ."</TD><TD></TD><TD><B>Domains File:</B> $domains_file</TD></TR>\n";
		print "<TR ALIGN=LEFT><TH>Source IP recorded:</TH><TD> ". keys(%s5) ."</TD><TD></TD><TD><B>Number of domains:</B> ". keys(%DomainName) ."</TD></TR>\n";
		print "<TR ALIGN=LEFT><TH>Destination IP recorded:</TH><TD> ". keys(%s6) ."</TD></TR>\n";
		print "<TR ALIGN=LEFT><TH>Preprocessor anomaly:</TH><TD> ". $anomaly ."</TD><TD></TD><TD><B>Rules File:</B> $rules_file</TD></TR>\n";
		print "<TR ALIGN=LEFT><TH>Severity recorded:</TH><TD> ". keys(%s20) ."</TD><TD></TD><TD><B>Number of referenced rules:</B> ". keys(%link) ."</TR>\n";
		print "<TR ALIGN=LEFT><TH>Portscan detected:</TH><TD> ", eval '$#posres +1',"</TD></TR>\n" if $opt_p;
		print "<TR ALIGN=LEFT><TH>Anomaly detected:</TH><TD> ", eval '$#anores +1',"</TD></TR>\n" if $opt_n;
		print "</TABLE>\n";
		print "<HR>\n";

	} else {
		print "the log begins from: $result[0]->[0] $result[0]->[1] $result[0]->[2]:$result[0]->[3]:$result[0]->[4]\n";
		print "the log ends     at: $result[$#result]->[0] $result[$#result]->[1] $result[$#result]->[2]:$result[$#result]->[3]:$result[$#result]->[4]\n";
		print "total events: $total\n";
		print "NIDS recorded: ". keys(%s13) ."\n";
		print "Signatures recorded: ". keys(%s4) ."\n";
		print "Classification recorded: ". keys(%s10) ."\n";
		print "Source IP recorded: ". keys(%s5) ."\n";
		print "Destination IP recorded: ". keys(%s6) ."\n";
		print "Preprocessor anomaly: ". $anomaly ."\n";
		print "Severity recorded: ". keys(%s20) ."\n";
		print "Portscan recorded: ", eval '$#posres +1',"\n" if $opt_p;
		print "Anomaly recorded: ", eval '$#anores +1',"\n" if $opt_n;
	}
}

# print menu for HTML page
sub print_menu {
	if ($opt{h}) {
		print "<li><a href=\"#attack_src\">To see the popularity of one source host</a>\n";
		print "<li><a href=\"#same_src_attack\">Percentage and number of attacks from one host to any with same method</a>\n";
		print "<li><a href=\"#attack_dst\">To see the popularity of one destination host</a>\n";
		print "<li><a href=\"#same_dst_attack\">Percentage and number of attacks to one host from any with same method</a>\n";
		print "<li><a href=\"#same_src_dst_attack\">Percentage and number of attacks from a host to a destination</a>\n";
		print "<li><a href=\"#attack\">The distribution of attack methods</a>\n";
		print "<li><a href=\"#same_class\">The distribution of classification method</a>\n";
		print "<li><a href=\"#severity\">The distribution of severity</a>\n";
		print "<li><a href=\"#daily_event\">The distribution of event by day</a>\n" if ($result[0]->[0] ne $result[$#result]->[0]) || ($result[0]->[1] < $result[$#result]->[1]);
		print "<li><a href=\"#hour\">The distribution of attack by hour</a>\n";
		print "<li><a href=\"#hour_attack\">Percentage and number of attacks by hour</a>\n";
		print "<li><a href=\"#port_attack\">Percentage and number of attacks to one destination port grouped by attack</a>\n";
		print "<li><a href=\"#nids\">The distribution of event by sonde</a>\n" if (keys(%s13) > 1);
		print "<li><a href=\"#stateful\">The distribution of stateful problems</a>\n" if ($anomaly > 0);
		print "<li><a href=\"#domain_src\">To see the popularity of one domain</a>\n" if ( $opt{c} );
		print "<li><a href=\"#portscan\">To see the portscan list</a>\n" if ( $totalscan > 0 );
		print "</ul><HR>\n";
	}
}       







sub daily_event {
$max = 0;
if ($opt{h}) {
	print "<h3><a name=\"daily_event\">The distribution of event by day</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>Jour</td><td>Mois</td><td>Nb</td><td>%</td><td>Graph</td></tr>";
	foreach $k (%s19) {
		$max = $s19{$k} if ($s19{$k} > $max);
	}
	my $ratio = $max/$total*100;
	foreach $k (sort keys %s19) {
		@_ = split ",",$k;
		my $num = $s19{$k}/$total*(50000/$ratio);
		printf("<tr class=TB><td>%-${hour_len}d</td><td>%-3s</td><td>%-${nb_len}d</td><td>%-2.2f</td><td><hr noshade align=left size=10 color=green  width=%3d ></td></tr>\n", $_[0],$_[1],$s19{$k},$s19{$k}/$total*100, $num);
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	foreach $k (%s19) {
		$max = $s19{$k} if ($s19{$k} > $max);
	}
	$ratio = $max/$total*100;
	section_header("The distribution of event by day\n", "daily_event");
	foreach $k (sort keys %s19) {
		@_ = split ",",$k;
		$num = $s19{$k}/$total*75*(100 / $ratio);
		printf("  %-${hour_len}d  %-2d     %-${nb_len}d  %5.2f  %-${graph_len}s\n", $_[0],$_[1],$s19{$k},$s19{$k}/$total*100, '#' x $num);
	}
}
}


sub severity {
$max = 0;
if ($opt{h}) {
	print "<h3><a name=\"severity\">The distribution of severity</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>Severity</td><td>Graph</td></tr>";
	foreach $k (%s20) {
		$max = $s20{$k} if ($s20{$k} > $max);
	}
	 my $ratio = $max/$total*100;
	foreach $k (sort { $s20{$b} <=> $s20{$a} } keys %s20) {
		my $num = $s20{$k}/$total*(50000/$ratio);
		$STYLE = init_color($k);
		printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${sever_len}s</td><td><hr noshade align=left size=10 color=green  width=%3d ></td></tr>\n", $s20{$k}/$total*100,$s20{$k},$k,$num);
	}
	print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	foreach $k (%s20) {
		$max = $s20{$k} if ($s20{$k} > $max);
	}
	$ratio = $max/$total*100;
	section_header("The distribution of severity\n", "severity");
	foreach $k (sort { $s20{$b} <=> $s20{$a} } keys %s20) {
		$num = $s20{$k}/$total*75*(100 / $ratio);
		printf("  %-5.2f  %-${nb_len}d  %-${sever_len}s  %-${graph_len}s\n", $s20{$k}/$total*100,$s20{$k},$k, '#' x $num);
	}
}
}



sub nids {
if ($opt{h}) {
	print "<h3><a name=\"nids\">The distribution of event by sonde</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>Sonde</td></tr>";
	foreach $k (sort { $s13{$b} <=> $s13{$a} } keys %s13) {
		printf("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td>%-32s</td></tr>\n", $s13{$k}/$total*100,$s13{$k},$k); 
	}
	print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("The distribution of event by sonde\n", "nids");
	foreach $k (sort { $s13{$b} <=> $s13{$a} } keys %s13) {
		printf("  %5.2f  %-${nb_len}d  %-32s\n", $s13{$k}/$total*100,$s13{$k},$k); 
	}
}
}

sub nids_attack {
$i=0;
   section_header("The distribution of event by sonde\n", "nids_attack");
   foreach $k (sort { $s14{$b} <=> $s14{$a} } keys %s14) {
      @_ = split ",",$k;
      printf("  %-5.2f  %-${nb_len}d  %-31s  %-${attack_len}s\n",
	     $s14{$k}/$total*100, $s14{$k},$_[0],$_[1]) if $i < $nb;
	     ++$i;
    }
}


sub stateful {
if ($opt{h}) {
	print "<h3><a name=\"stateful\">The distribution of stateful problems</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>Processor</td></tr>";
	foreach $k (sort { $s17{$b} <=> $s17{$a} } keys %s17) {
		printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-32s</td></tr>", $s17{$k}/$total*100,$s17{$k},$k) if $k ne "none"; 
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("The distribution of stateful problems\n", "stateful");
	foreach $k (sort { $s17{$b} <=> $s17{$a} } keys %s17) {
		printf("  %5.2f  %-${nb_len}d  %-32s\n", $s17{$k}/$total*100,$s17{$k},$k) if $k ne "none"; 
	}
}
}


sub same_src_dst_attack {
$i=0;
if ($opt{h}) {
	print "<h3><a name=\"same_src_dst_attack\">Percentage and number of attacks from a host to a destination</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td><td>IP Destination</td><td>Attack</td></tr>";
	if ($opt{i}) {
		foreach $k (sort { $s0{$a} <=> $s0{$b} } keys %s0) { 
			@_ = split ",",$k;
			printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td></tr>\n", $s0{$k}/$total*100, $s0{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	} else {
		foreach $k (sort { $s0{$b} <=> $s0{$a} } keys %s0) { 
			@_ = split ",",$k;
			printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td></tr>\n", $s0{$k}/$total*100, $s0{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("The number of attacks from same host to same destination using same method\n", "same_src_dst_attack");
	if ($opt{i}) {
		foreach $k (sort { $s0{$a} <=> $s0{$b} } keys %s0) { 
			@_ = split ",",$k;
			printf("  %-5.2f  %-${nb_len}d  %-${addr_len}s  %-${addr_len}s  %-${attack_len}s\n",
			$s0{$k}/$total*100, $s0{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	} else {
		foreach $k (sort { $s0{$b} <=> $s0{$a} } keys %s0) { 
			@_ = split ",",$k;
			printf("  %-5.2f  %-${nb_len}d  %-${addr_len}s  %-${addr_len}s  %-${attack_len}s\n",
			$s0{$k}/$total*100, $s0{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	}
}
}


# to see how many attacks launched from one host
sub same_src_attack {
$i=0;
if ($opt{h}) {
	print "<h3><a name=\"same_src_attack\">Percentage and number of attacks from one host to any with same method</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td><td>Attack</td><td>Severity</td></tr>";
	if ($opt{i}) {
		foreach $k (sort { $s2{$a} <=> $s2{$b} } keys %s2) {
			@_ = split ",",$k;
			$STYLE = init_color($_[2]);
			printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td><td>%-${server_len}s</td></tr>\n", $s2{$k}/$total*100, $s2{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i
		}
	} else {
		foreach $k (sort { $s2{$b} <=> $s2{$a} } keys %s2) {
			@_ = split ",",$k;
			$STYLE = init_color($_[2]);
			printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td><td>%-${server_len}s</td></tr>\n", $s2{$k}/$total*100, $s2{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("Percentage and number of attacks from one host to any with same method\n", "same_src_attack");  
	if ($opt{i}) {
		foreach $k (sort { $s2{$a} <=> $s2{$b} } keys %s2) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${attack_len}s  %-${server_len}s\n",
			$s2{$k}/$total*100, $s2{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i
		}
	} else {
		foreach $k (sort { $s2{$b} <=> $s2{$a} } keys %s2) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${attack_len}s  %-${sever_len}s\n",
			$s2{$k}/$total*100, $s2{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	}
}
}


# to see how many attacks received by one host (destination correlated)
sub same_dst_attack {
$i=0;
if ($opt{h}) {
	print "<h3><a name=\"same_dst_attack\">Percentage and number of attacks to one host from any with same method</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td><td>Attack</td><td>Severity</td></tr>";
	if ($opt{i}) {
		foreach $k (sort { $s3{$a} <=> $s3{$b} } keys %s3) {
			@_ = split ",",$k;
			$STYLE = init_color($_[2]);
			printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td><td>%-${sever_len}s</td></tr>\n",$s3{$k}/$total*100 , $s3{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	} else {
		foreach $k (sort { $s3{$b} <=> $s3{$a} } keys %s3) {
			@_ = split ",",$k;
			$STYLE = init_color($_[2]);
			printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td><td>%-${sever_len}s</td></tr>\n",$s3{$k}/$total*100 , $s3{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("Percentage and number of attacks to one host from any with same method\n", "same_dst_attack");
	if ($opt{i}) {
		foreach $k (sort { $s3{$a} <=> $s3{$b} } keys %s3) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${attack_len}s  %-${sever_len}s\n",$s3{$k}/$total*100 ,
			$s3{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	} else {
		foreach $k (sort { $s3{$b} <=> $s3{$a} } keys %s3) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${attack_len}s  %-${sever_len}s\n",$s3{$k}/$total*100 ,
			$s3{$k},$_[1],$_[0],$_[2]) if $i < $nb;
			++$i;
		}
	}
}
}


# to see the popularity of one attack method
sub attack {
$i=0;
if ($opt{h}) {
	print "<h3><a name=\"attack\">The distribution of attack methods ( ". $nb ." of ". keys(%s4) ." )</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>Attack</td><td>Priority</td><td>Severity</td></tr>";
	if ($opt{i}) {
		foreach $k (sort { $s4{$a} <=> $s4{$b} } keys %s4) {
			@_ = split ",",$k;
			if ($link{$_[0]} =~ /(^.+),(.*$)/) {
				$url = init_url($1);
				printf("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td><a href=$url$2>%-${attack_len}s</a></td><td>%-${prior_len}d</td><td>%-${sever_len}s</td><td>%-20s</td></tr>\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]) if $i < $nb; 
				++$i;
			} else {
				printf("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td>%-${attack_len}s</td><td>%-${prior_len}d</td><td>%-${sever_len}s</td><td>%-20s</td></tr>\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]) if $i < $nb; 
				++$i;
			}
		}
	} else {
		foreach $k (sort { $s4{$b} <=> $s4{$a} } keys %s4) {
			@_ = split ",",$k;
			if ($link{$_[0]} =~ /(^.+),(.*$)/) {
				$url = init_url($1);
				printf("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td><a href=$url$2>%-${attack_len}s</a></td><td>%-${prior_len}d</td><td>%-${sever_len}s</td></tr>\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]) if $i < $nb; 
				++$i;
			} else {
				printf("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td>%-${attack_len}s</td><td>%-${prior_len}d</td><td>%-${sever_len}s</td></tr>\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]) if $i < $nb; 
				++$i;
			}
		}
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("The distribution of attack methods\n", "attack");
	if ($opt{i}) {
		foreach $k (sort { $s4{$a} <=> $s4{$b} } keys %s4) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${attack_len}s  %-${prior_len}d %-${sever_len}s\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]) if $i < $nb; 
			++$i;
		}
	} else {
		foreach $k (sort { $s4{$b} <=> $s4{$a} } keys %s4) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${attack_len}s  %-${prior_len}d %-${sever_len}s\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]) if $i <= $nb; 
			++$i;
		}
	}
}
}


# to see the popularity of one source host 
sub same_class {
if ($opt{h}) {
	print "<h3><a name=\"same_class\">The distribution of classification method</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>Classification</td><td>Severity</td></tr>";
	if ($opt{i}) {
		foreach $k (sort { $s10{$a} <=> $s10{$b} } keys %s10) {
			@_ = split ",",$k;
			$STYLE = init_color($_[1]);
			printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${class_len}s</td><td>%-${server_len}s</td></tr>\n", $s10{$k}/$total*100,$s10{$k},$_[0],$_[1]); 
		}
	} else {
		foreach $k (sort { $s10{$b} <=> $s10{$a} } keys %s10) {
			@_ = split ",",$k;
			$STYLE = init_color($_[1]);
			printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${class_len}s</td><td>%-${server_len}s</td></tr>\n", $s10{$k}/$total*100,$s10{$k},$_[0],$_[1]); 
		}
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("The distribution of classification method\n", "same_class");
	if ($opt{i}) {
		foreach $k (sort { $s10{$a} <=> $s10{$b} } keys %s10) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${class_len}s  %-${server_len}s\n", $s10{$k}/$total*100,$s10{$k},$_[0],$_[1]); 
		}
	} else {
		foreach $k (sort { $s10{$b} <=> $s10{$a} } keys %s10) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${class_len}s  %-${sever_len}s\n", $s10{$k}/$total*100,$s10{$k},$_[0],$_[1]); 
		}
	}
}
}



# to see the popularity of one source host 
sub attack_src {
$i=0;

if ($opt{h}) {
	print "<h3><a name=\"attack_src\">To see the popularity of one source host ( ". $nb ." of ". keys(%s5) ." )</a></h3>\n";
	print "<table border=5>\n";
	if ($opt{r}) {
		print"<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td><td>Resolve</td><td>Domain</td></tr>";
	} else {
		print"<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td></tr>";
	}
	
	if ($opt{i}) {
		foreach $k (sort { $s5{$a} <=> $s5{$b} } keys %s5) {
			if ($i < $nb) {
				if ($opt{r}) {
					$host = resolve ($k);
					$src_host = $host if ($host ne $k);
					$STYLE = init_color($src_host);
					$src_host =~ /(\w+$)/;
					printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${addr_len}s</td><td>%-${domain_len}s</td></tr>\n",$s5{$k}/$total*100,$s5{$k},$k,$src_host,$DomainName{$1});
				} else {
					printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td></tr>\n", $s5{$k}/$total*100,$s5{$k},$k);
				}
			}
		++$i;
		}
	} else {
		foreach $k (sort { $s5{$b} <=> $s5{$a} } keys %s5) {
			if ($i < $nb) {
				if ($opt{r}) {
					$host = resolve ($k);
					$src_host = $host if ($host ne $k);
					$STYLE = init_color($src_host);
					$src_host =~ /(\w+$)/;
					printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${resolve_len}s</td><td>%-${domain_len}s</td></tr>\n",$s5{$k}/$total*100,$s5{$k},$k,$src_host,$DomainName{$1});
				} else {
					 printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td></tr>\n", $s5{$k}/$total*100,$s5{$k},$k);
				}
			}
		++$i;
		}
	}
	print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	if ($opt{r}) {
		section_header("To see the popularity of one source host\n", "attack_src_resolve");
	} else {
		section_header("To see the popularity of one source host\n", "attack_src");
	}
	if ($opt{i}) {
		foreach $k (sort { $s5{$a} <=> $s5{$b} } keys %s5) {
			if ($i < $nb) {
				if ($opt{r}) {
					$host = resolve ($k);
					$src_host = $host if ($host ne $k);
					$src_host =~ /(\w+$)/;
					$s40{"$1"}++;
					printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${resolve_len}s  %-${domain_len}s\n", $s5{$k}/$total*100,$s5{$k},$k,$src_host,$DomainName{$1});
				} else {
					printf("  %5.2f  %-${nb_len}d  %-${addr_len}s\n", $s5{$k}/$total*100,$s5{$k},$k) if $i < $nb; 
				}
			}
		++$i;
		}
	} else {
		foreach $k (sort { $s5{$b} <=> $s5{$a} } keys %s5) {
			if ($i < $nb) {
				if ($opt{r}) {
					$host = resolve ($k);
					$src_host = $host if ($host ne $k);
					$src_host =~ /(\w+$)/;
#					$s40{"$1"}++;
					printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${resolve_len}s  %-${domain_len}s\n", $s5{$k}/$total*100,$s5{$k},$k,$src_host,$DomainName{$1});
				} else {
					printf("  %5.2f  %-${nb_len}d  %-${addr_len}s\n", $s5{$k}/$total*100,$s5{$k},$k);
				}
			}
		++$i;
		}
	}

}
}




sub domain_src {
$i=0;

if ($opt{h}) {
	print "<h3><a name=\"domain_src\">To see the popularity of one domain ( Top : ". $nb ." )</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>Domain</td></tr>";
	foreach $k (sort { $s5{$b} <=> $s5{$a} } keys %s5) {
		$host = resolve ($k);
		$src_host = $host if ($host ne $k);
		$src_host =~ /(\w+$)/;
		$s40{"$1"}++;
	}
	if ($opt{i}) {
		foreach $l (sort { $s40{$a} <=> $s40{$b} } keys %s40) {
			$STYLE = init_color($l);
			printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${domain_len}s</td></tr>\n", $s40{$l}/(keys %s5)*100,$s40{$l},$DomainName{$l}) if $i < $nb;
			++$i;
		}
	} else {
		foreach $l (sort { $s40{$b} <=> $s40{$a} } keys %s40) {
			$STYLE = init_color($l);
			printf("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${domain_len}s</td></tr>\n", $s40{$l}/(keys %s5)*100,$s40{$l},$DomainName{$l}) if $i < $nb;
			++$i;
		}
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	foreach $k (sort { $s5{$b} <=> $s5{$a} } keys %s5) {
		$host = resolve ($k);
		$src_host = $host if ($host ne $k);
		$src_host =~ /(\w+$)/;
		$s40{"$1"}++;
	}
	section_header("To see the popularity of one domain\n", "domain_src");
	if ($opt{i}) {
		foreach $l (sort { $s40{$a} <=> $s40{$b} } keys %s40) {
			printf("  %5.2f  %-${nb_len}d  %-${domain_len}s\n", $s40{$l}/(keys %s5)*100,$s40{$l},$DomainName{$l}) if $i < $nb;
			++$i;
		}
	} else {
		foreach $l (sort { $s40{$b} <=> $s40{$a} } keys %s40) {
			printf("  %5.2f  %-${nb_len}d  %-${domain_len}s\n", $s40{$l}/(keys %s5)*100,$s40{$l},$DomainName{$l}) if $i < $nb;
			++$i;
		}
	}
}
}



# to see the popularity of one destination host
sub attack_dst {
$i=0;

if ($opt{h}) {
	print "<h3><a name=\"attack_dst\">To see the popularity of one destination host ( ". $nb ." of ". keys(%s6) ." )</a></h3>\n";
	print "<table border=5>\n";
	if ($opt{r}) {
		print"<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Destination</td><td>Resolve</td></tr>";
	} else {
		print"<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Destination</td></tr>";
	}

	if ($opt{i}) {
		foreach $k (sort { $s6{$a} <=> $s6{$b} } keys %s6) {
			if ($opt{r}) {
				$host = resolve ($k);
				$src_host = $host if ($host ne $k);
				printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${resolve_len}s</td></tr>\n",$s6{$k}/$total*100,$s6{$k},$k,$src_host) if $i < $nb;
			} else {
				printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td></tr>\n", $s6{$k}/$total*100,$s6{$k},$k) if $i < $nb;
			}
		++$i;
		}
	} else {
		foreach $k (sort { $s6{$b} <=> $s6{$a} } keys %s6) {
			if ($opt{r}) {
				$host = resolve ($k);
				$src_host = $host if ($host ne $k);
				printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${resolve_len}s</td></tr>\n",$s6{$k}/$total*100,$s6{$k},$k,$src_host) if $i < $nb;
			} else {
				printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td></tr>\n", $s6{$k}/$total*100,$s6{$k},$k) if $i < $nb;
			}
		++$i;
		}
	}
	print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	if ($opt{r}) {
		section_header("To see the popularity of one destination host\n", "attack_dst_resolve");
	} else {
		section_header("To see the popularity of one source host\n", "attack_dst");
	}
	if ($opt{i}) {
		foreach $k (sort { $s6{$a} <=> $s6{$b} } keys %s6) {
			if ($opt{r}) {
				$host = resolve ($k);
				$src_host = $host if ($host ne $k);
				printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${resolve_len}s\n", $s6{$k}/$total*100,$s6{$k},$k,$src_host) if $i < $nb;
			} else {
				printf("  %5.2f  %-${nb_len}d  %-${addr_len}s\n", $s6{$k}/$total*100,$s6{$k},$k) if $i < $nb; 
			}
		++$i;
		}
	} else {
		foreach $k (sort { $s6{$b} <=> $s6{$a} } keys %s6) {
			if ($opt{r}) {
				$host = resolve ($k);
				$src_host = $host if ($host ne $k);
				printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${resolve_len}s\n", $s6{$k}/$total*100,$s6{$k},$k,$src_host) if $i < $nb;
			} else {
				printf("  %5.2f  %-${nb_len}d  %-${addr_len}s\n", $s6{$k}/$total*100,$s6{$k},$k) if $i < $nb; 
			}
		++$i;
		}
	}

}
}



# to see the activity by hour 
sub hour {
$max = 0;
if ($opt{h}) {
	print "<h3><a name=\"hour\">The distribution of attack by hour</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>Hour</td><td>Nb</td><td>%</td><td>Graph</td></tr>";
	foreach $k (%s7) {
		$max = $s7{$k} if ($s7{$k} > $max);
	}
	 my $ratio = $max/$total*100;
	foreach $k (sort keys %s7) {
		my $num = $s7{$k}/$total*(50000/$ratio);
		printf("<tr class=TB><td>%${hour_len}dh</td><td>%-${nb_len}d</td><td>%-2.2f</td><td><hr noshade align=left size=10 color=#FFFFFF  width=%3d ></td></tr>\n", $k,$s7{$k},$s7{$k}/$total*100,$num); 
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	foreach $k (%s7) {
		$max = $s7{$k} if ($s7{$k} > $max);
	}
	$ratio = $max/$total*100;
	section_header("The distribution of attack by hour\n", "hour");
	foreach $k (sort keys %s7) {
		my $num = $s7{$k}/$total*75*(100 / $ratio);
		printf("  %${hour_len}dh  %-${nb_len}d  %5.2f  %-${graph_len}s\n", $k,$s7{$k},$s7{$k}/$total*100, '#' x $num); 
	}
}
}


sub same_hour_attack {
$i=0;
if ($opt{h}) {
	print "<h3><a name=\"hour_attack\">Percentage and number of attacks by hour</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>Hour</td><td>Attack</td></tr>";
	if ($opt{i}) {
		foreach $k (sort { $s9{$a} <=> $s9{$b} } keys %s9) {
			@_ = split ",",$k;
			printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%${hour_len}dh</td><td>%-${attack_len}s</td></tr>\n",
			$s9{$k}/$total*100, $s9{$k},$_[0],$_[1]) if $i < $nb;
			++$i
		}
	} else {
		foreach $k (sort { $s9{$b} <=> $s9{$a} } keys %s9) {
			@_ = split ",",$k;
			printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%${hour_len}dh</td><td>%-${attack_len}s</td></tr>\n",
			$s9{$k}/$total*100, $s9{$k},$_[0],$_[1]) if $i < $nb;
			++$i;
		}
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("Percentage and number of attacks by hour\n", "same_hour_attack");  
	if ($opt{i}) {
		foreach $k (sort { $s9{$a} <=> $s9{$b} } keys %s9) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %${hour_len}dh    %-${attack_len}s\n",
			$s9{$k}/$total*100, $s9{$k},$_[0],$_[1]) if $i < $nb;
			++$i
		}
	} else {
		foreach $k (sort { $s9{$b} <=> $s9{$a} } keys %s9) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %${hour_len}dh    %-${attack_len}s\n",
			$s9{$k}/$total*100, $s9{$k},$_[0],$_[1]) if $i < $nb;
			++$i;
		}
	}
}
}


sub dport_attack {
$i = 0;
if ($opt{h}) {
	print "<h3><a name=\"port_attack\">Percentage and number of attacks to one destination port grouped by attack</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>Port</td><td>Attack</td></tr>";
	if ($opt{i}) {
		foreach $k (sort { $s18{$a} <=> $s18{$b} } keys %s18) {
			@_ = split ",",$k;
			printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${port_len}s</td><td>%-${attack_len}s</td></tr>\n",
			$s18{$k}/$total*100, $s18{$k},$_[0],$_[1]) if ( $i < $nb );
			++$i;
		}
	} else {
		foreach $k (sort { $s18{$b} <=> $s18{$a} } keys %s18) {
			@_ = split ",",$k;
			printf("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${port_len}s</td><td>%-${attack_len}s</td></tr>\n",
			$s18{$k}/$total*100, $s18{$k},$_[0],$_[1]) if ( $i < $nb );
			++$i;
		}
	}
print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("Percentage and number of attacks to one destination port grouped by attack\n", "dport_attack");  
	if ($opt{i}) {
		foreach $k (sort { $s18{$a} <=> $s18{$b} } keys %s18) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${port_len}s  %-${attack_len}s\n",
			$s18{$k}/$total*100, $s18{$k},$_[0],$_[1]) if ( $i < $nb );
			++$i;
		}
	} else {
		foreach $k (sort { $s18{$b} <=> $s18{$a} } keys %s18) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${port_len}s  %-${attack_len}s\n",
			$s18{$k}/$total*100, $s18{$k},$_[0],$_[1]) if ( $i < $nb );
			++$i;
		}
	}
}
}


sub init_domains {
open(DOMAINS,$domains_file) or die "Can not open domain file: $domains_file $!\n";
$MaxDomain=0;
while (<DOMAINS>) {
	next if (/^$/ or /^#/);
	$Dots=1;
	($Code,$Description) = /(^\S+)\s+(.*)/;
	$Code =~ tr/A-Z/a-z/;
	$DomainName{$Code}="$Description";
	$Dots++ while $Code =~ /\./g;
	$MaxDomain=$Dots if ($Dots >$MaxDomain);
}
close(DOMAINS);
}



sub init_rules {
open(RULES,$rules_file) or die "Can not open domain file: $rules_file $!\n";
while (<RULES>) {
next if (/^$/ or /^#/);
($descr_attack,$link_attack) = /(^.+})\s+(.*)$/;
$link{$descr_attack}="$link_attack";
}
close(RULES);
}



sub init_color {
my $LOG = shift;
my $COLOR;
if (($LOG eq "high") || ($LOG =~ /nresolved/)) {
	$COLOR="TBR";
} elsif ($LOG eq "unknown") {
	$COLOR="TBG";
} else {
	$COLOR="TB";
}
return ($COLOR)
}




sub init_url {
my $LOG = shift;
my $URL;
if ($LOG eq "bugtraq") {
	$URL="http://www.securityfocus.com/bid/";
} elsif ($LOG eq "arachnids") {
	$URL="http://www.whitehats.com/info/IDS";
} elsif ($LOG eq "cve") {
	$URL="http://cve.mitre.org/cgi-bin/cvename.cgi?name=";
} elsif ($LOG eq "nessus") {
	$URL="http://cgi.nessus.org/plugins/dump.php3?id=";
} elsif ($LOG eq "MCAFEE") {
	$URL="http://vil.nai.com/vil/content/v_";
} elsif ($LOG eq "url") {
	$URL="http://";
}
return ($URL);
}

# portscan (if enable -p switch)
# Please use '-A fast' to generate the log, so portscan() can process it.
# contributed by: Paul Bobby, <paul.bobby@lmco.com>
#                 Jian-Da Li, <jdli@freebsd.csie.nctu.edu.tw>
sub portscan {
my (%s1, %s2);
# to see how many times a host performs portscan
# used in portscan()
for $i (0 .. $#portscan) {
	$s1{"$portscan[$i]->[0]"}++;
}

if ( $opt{h} ) {
	print "<h3><a name=\"portscan\">Portscan List</a></h3>\n";
	print "<table border=5>\n";
	print"<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td></tr>";
	foreach $k (sort { $s1{$b} <=> $s1{$a} } keys %s1) {
		printf("<tr class=TB><TD>%-2.2f</TD><TD>%-${nb_len}d</TD><TD>%-${addr_len}s\n", $s1{$k}/$totalscan*100,$s1{$k},$k);
	}
	print "</table><a href=\"#top\">Top</a><hr>\n";
} else {
	section_header("Portscans performed to/from HOME_NET\n", "portscan");
	foreach $k (sort { $s1{$b} <=> $s1{$a} } keys %s1) {
		printf("  %5.2f  %-4d   %-${addr_len}s\n", $s1{$k}/$totalscan*100,$s1{$k},$k);
	}
}
}

# anomsensor (if enable -n switch)
# This function process data generated by spp_anomsensor plug-in (SPADE)
# By Yen-Ming Chen <chenym@alumni.cmu.edu>
sub anomsensor {
my (%s7);
# to see how many times a host performs portscan
# used in anomsensor()
for $i (0 .. $#anores) {
$s7{"$anores[$i]->[1],$anores[$i]->[3],$anores[$i]->[4]"}++;
}
section_header("Anomaly detected by SPADE\n", "asdo");
foreach $k (sort { $s7{$b} <=> $s7{$a} } keys %s7) {
@_ = split(/,/,$k);
printf("   %-4d   %-${addr_len}s %-${addr_len}s\t%-6d\n", $s7{$k},$_[0],$_[1],$_[2]) if $s7{$k};
}
}

# print the footer (needed for html)
sub print_footer {
if ($opt{h}) {
	print "Generated by <a href=http://jeremy.chartier.free.fr/snort-ng/>SNORT-NG</a> Version: $version (Date: $datever )\n";
	print "</BODY>\n</HTML>";
} else {
print <<FootMessage

Version: $version
Jrmy CHARTIER, <jeremy.chartier\@free.fr>
Date: $datever
FootMessage
}
}


#
# resolve host name and cache it
#
sub resolve {

my $Address=shift;
my $Hostname;

if ($Address =~ /(\d+\.\d+\.\d+\.\d+)/) {
$Hostname = gethostbyaddr(inet_aton($Address),AF_INET) or $Hostname="unresolved";
}
return $Hostname;
}








# Use a title and a short code to write the section headers
# This is used in place of a FORMAT as this allows variable column widths
# contributed by: Ned Patterson, <jpatter@alum.mit.edu>
#
sub section_header {
my $linelength;
$title = shift; 
$report = shift;
$_ = shift;
print("\n\n$title");
# constant for method length for now
#  $linelenght = (/p/?7:0) + (/a/?20:0) + (/s/?$addr_len:0) +
#(/d/?$addr_len+3:0) + (/m/?20:0);
#$linelength = 36;
#  print( '=' x $linelength, "\n");

if ( $report eq "attack_src_resolve") {
$linelength = 11 + $nb_len + 2 + $addr_len + 2 + $resolve_len + 2 + $domain_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s5);
}
print(" " x 4, "### ". $nb ." of ". keys(%s5) ." ###\n");
print("    %    nb      IP source        Resolve                                             Domain\n");
} elsif ($report eq "domain_src") {
$linelength = 11 + $domain_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s6);
}
print(" " x 4, "### ". $nb ." of ". keys(%s40) ." ###\n");
print("    %    nb      Domain\n");
} elsif ($report eq "attack_dst_resolve") {
$linelength = 11 + $nb_len + 2 + $addr_len + 2 + $resolve_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s6);
}
print(" " x 4, "### ". $nb ." of ". keys(%s6) ." ###\n");
print("    %    nb      IP destination   Resolve\n");
} elsif ($report eq "attack") {
$linelength = 13 + $nb_len + 4 + $attack_len + 4 + $prior_len + 4 + $sever_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s4);
}
print(" " x 4, "### ". keys(%s4) ." of ". keys(%s4) ." ###\n");
print("    %    nb      Attack                                                                               Priority Severity\n");
} elsif ($report eq "stateful") {
$linelength = 40;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s17);
}
print(" " x 4, "### ". $anomaly ." of ". $anomaly ." ###\n");
print("    %    nb      Preprocessor\n");
#} elsif ($report eq "port") {
#$linelength = 30;
#print( '=' x $linelength, "\n");
#if ($opt{n}) {
#$nb = $opt{n};
#} else {
#$nb = keys(%s8);
#}
#print(" " x 4, "### ". keys(%s8) ." of ". keys(%s8) ." ###\n");
#print("    %      nb        Port\n");
} elsif ($report eq "same_class") {
$linelength = 13 + $nb_len + 4 + $class_len + 4 + $sever_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s10);
}
print(" " x 4, "### ". keys(%s10) ." of ". keys(%s10) ." ###\n");
print("    %    nb       Classification                                               Severity\n");
} elsif ($report eq "same_hour_attack") {
$linelength = 11 + $nb_len + 4 + $hour_len + 4 + $attack_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s9);
}
print(" " x 4, "### ". $nb ." of ". keys(%s9) ." ###\n");
print("    %    nb      Heure  Attack\n");
} elsif ($report eq "same_src_attack") {
$linelength = 11 + $nb_len + 4 + $addr_len + 4 + $attack_len + 4 + $sever_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s2);
}
print(" " x 4, "### ". $nb ." of ". keys(%s2) ." ###\n");
print("    %    nb      IP source        Attack                                                                                     Severity\n");
} elsif ($report eq "same_dst_attack") {
$linelength = 13 + $nb_len + 4 + $addr_len + 4 + $attack_len + 4 + $sever_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s3);
}
print(" " x 4, "### ". $nb ." of ". keys(%s3) ." ###\n");
print("    %    nb      IP destination   Attack                                                                                      Severity\n");
} elsif ($report eq "same_src_dst_attack") {
$linelength = 13 + $nb_len + 4 + $addr_len + 4 + $addr_len + 4 + $attack_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s0);
}
print(" " x 4, "### ". $nb ." of ". keys(%s0) ." ###\n");
print("    %    nb      IP source        IP destination   Attack\n");
} elsif ($report eq "dport_attack") {
$linelength = 11 + $nb_len + 4 + $port_len + 4 +  $attack_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s18);
}
print(" " x 4, "### ". $nb ." of ". keys(%s18) ." ###\n");
print("    %    nb      Port   Attack\n");
} elsif ($report eq "daily_event") {
$linelength = 20 + $nb_len + 6 + $graph_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s19);
}
print(" " x 4, "### ". keys(%s19) ." of ". keys(%s19) ." ###\n");
print(" Day Month  nb         %    Graph\n");
} elsif ($report eq "hour") {
$linelength = 13 + $nb_len + 2 + $hour_len + 2 + $graph_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s7);
}
print(" " x 4, "### ". keys(%s7) ." of ". keys(%s7) ." ###\n");
print("  Hour nb        %    Graph\n");
} elsif ($report eq "portscan") {
$linelength = 40;
print( '=' x $linelength, "\n");
print(" " x 4, "### Portscan List ###\n");
print("    %    nb      IP destination\n");
} elsif ($report eq "attack_dst") {
$linelength = 13 + $nb_len + 4 + $addr_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s6);
}
print(" " x 4, "### ". $nb ." of ". keys(%s7) ." ###\n");
print("    %    nb      IP destination\n");
} elsif ($report eq "attack_src") {
$linelength = 13 + $nb_len + 4 + $addr_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s6);
}
print(" " x 4, "### ". $nb ." of ". keys(%s5) ." ###\n");
print("    %    nb      IP source\n");
} elsif ($report eq "severity") {
$linelength = 13 + $nb_len + 2 + $sever_len + 2 + $graph_len;
print( '=' x $linelength, "\n");
if ($opt{n}) {
$nb = $opt{n};
} else {
$nb = keys(%s20);
}
print(" " x 4, "### ". keys(%s20) ." of ". keys(%s20) ." ###\n");
print("    %    nb      Severity Graph\n");
}
print( '=' x $linelength, "\n");
}



# Put data $alert into matrix for further process
# INPUT: $alert
sub process_data() {
$self = shift;
# put those data into a big matrix
if ($self->{PLUGIN} eq "anomsensor") {
push @anores , [$self->{THR},$self->{SADDR},$self->{SPORT},
	    $self->{DADDR},$self->{DPORT}];
#$opt_n = 1;
} elsif ($self->{PLUGIN} eq "portscan") {
	push @portscan , [$self->{SADDR}];
#$opt_p = 1;
} elsif ($self->{TYPE} eq "sys" || $self->{TYPE} eq "alert" || 
	$self->{PLUGIN} eq "stream4" || $self->{PLUGIN} eq "bo" ||
	$self->{PLUGIN} eq "frag2" || $self->{PLUGIN} eq "snort_decoder" ) {
	$self->{SIG} =~ s/\:$//o;
	push @result , [$self->{MON},$self->{DAY},$self->{HOUR},$self->{MIN},
                   $self->{SEC},$self->{HOST},$self->{SIG},$self->{SADDR},
                   $self->{SPORT},$self->{DADDR},$self->{DPORT},$self->{CLASS},
		   $self->{PLUGIN}, $self->{PRIORITY}, $self->{SEVERITY}];
    $lastwassnort = 1;
  } else {
    print STDERR "Unknown alert type plugin! $self->{TYPE}:$self->{PLUGIN} Skipped!\n" if $opt{d};
    return;
  }
#  1;
}


sub usage {
print <<Helpmsg
Usage: $0 <stats options> <options>

Options:
-i 			Inverse the result	
-r 			Resolve IP adresses	
-c 			Resolve domains	
-d 			Mode debug	
-n <integer>		For specify a number of line in the result
-help			View this help

Reports:
-src                      Top IPs sources
-dst                      Top IPs destination
-src_attack               Top IPs sources grouped by attack
-dst_attack               Top IPs destination grouped by attack
-src_dst_attack           Top alert grouped by IPs sources, Ips destination and attack
-attack                   Top attack
-class                    Top classification
-severity                 Top severity
-daily_event              Top number of attack grouped by day
-hour                     Top number of attack grouped by hour
-hour_attack              Top specific attack grouped by hour
-dport_attack             Top destination port grouped by attack
-nids                     Top NIDS host
-stateful                 Top stateful problems
-domain_src               Top of domain source
-portscan                 Top of portscan alert

print_footer();
Helpmsg
;

	exit 0;
}

